package NeuralNetwork;

import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.dataset.DataSet;
import org.nd4j.linalg.factory.Nd4j;
import org.nd4j.linalg.util.FeatureUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * Taken from deeplearning4J by Maaarty on 7.3.2017.
 */
public abstract class BaseDataFetcher implements DataSetFetcher {
    private static final long serialVersionUID = -859588773699432365L;
    protected int cursor = 0;
    protected int numOutcomes = -1;
    protected int inputColumns = -1;
    protected DataSet curr;
    protected int totalExamples;
    protected static final Logger log = LoggerFactory.getLogger(org.deeplearning4j.datasets.fetchers.BaseDataFetcher.class);

    public BaseDataFetcher() {
    }

    protected INDArray createInputMatrix(int numRows) {
        return Nd4j.create(numRows, this.inputColumns);
    }

    protected INDArray createOutputVector(int outcomeLabel) {
        return FeatureUtil.toOutcomeVector(outcomeLabel, this.numOutcomes);
    }

    protected INDArray createOutputMatrix(int numRows) {
        return Nd4j.create(numRows, this.numOutcomes);
    }

    protected void initializeCurrFromList(List<DataSet> examples) {
        if(examples.isEmpty()) {
            log.warn("Warning: empty dataset from the fetcher");
        }

        this.curr = null;
        INDArray inputs = this.createInputMatrix(examples.size());
        INDArray labels = this.createOutputMatrix(examples.size());

        for(int i = 0; i < examples.size(); ++i) {
            INDArray data = ((DataSet)examples.get(i)).getFeatureMatrix();
            INDArray label = ((DataSet)examples.get(i)).getLabels();
            inputs.putRow(i, data);
            labels.putRow(i, label);
        }

        this.curr = new DataSet(inputs, labels);
        examples.clear();
    }

    public void setLabelNames(List<String> names) {
        this.curr.setLabelNames(names);
    }

    public String getLabelName(int i) {
        return (String)this.curr.getLabelNames().get(i);
    }

    public boolean hasMore() {
        return this.cursor < this.totalExamples;
    }

    public DataSet next() {
        return this.curr;
    }

    public int totalOutcomes() {
        return this.numOutcomes;
    }

    public int inputColumns() {
        return this.inputColumns;
    }

    public int totalExamples() {
        return this.totalExamples;
    }

    public void reset() {
        this.cursor = 0;
    }

    public int cursor() {
        return this.cursor;
    }
}
