package NeuralNetwork;

import org.nd4j.linalg.dataset.DataSet;
import org.nd4j.linalg.dataset.api.DataSetPreProcessor;
import org.nd4j.linalg.dataset.api.iterator.DataSetIterator;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Maaarty on 7.3.2017.
 */
public class BaseDatasetIterator implements DataSetIterator {
    protected int batch;
    protected int numExamples;
    protected GomokuDataFetcher fetcher;
    protected DataSetPreProcessor preProcessor;

    public BaseDatasetIterator(int batch, int numExamples, BaseDataFetcher fetcher) {
        this.batch = batch;
        if (numExamples < 0) {
            numExamples = fetcher.totalExamples();
        }

        this.numExamples = numExamples;
        this.fetcher = (GomokuDataFetcher) fetcher;
    }

    public boolean hasNext() {
        return this.fetcher.hasMore() && this.fetcher.cursor() < this.numExamples;
    }

    public DataSet next() {
        this.fetcher.fetch(this.batch);
        DataSet result = this.fetcher.next();
        if (this.preProcessor != null) {
            this.preProcessor.preProcess(result);
        }

        return result;
    }

    public DataSet next(int num) {
        this.fetcher.fetch(num);
        DataSet next = this.fetcher.next();
        if (this.preProcessor != null) {
            this.preProcessor.preProcess(next);
        }

        return next;
    }

    public void remove() {
        throw new UnsupportedOperationException();
    }

    public int totalExamples() {
        return this.fetcher.totalExamples();
    }

    public int inputColumns() {
        return this.fetcher.inputColumns();
    }

    public int totalOutcomes() {
        return this.fetcher.totalOutcomes();
    }

    public boolean resetSupported() {
        return true;
    }

    public boolean asyncSupported() {
        return true;
    }

    public void reset() {
        this.fetcher.reset();
    }

    public int batch() {
        return this.batch;
    }

    public int cursor() {
        return this.fetcher.cursor();
    }

    public int numExamples() {
        return this.numExamples;
    }



    public void setPreProcessor(DataSetPreProcessor preProcessor) {
        this.preProcessor = preProcessor;
    }

    public List<String> getLabels() {
       return null;
    }

    public DataSetPreProcessor getPreProcessor() {
        return this.preProcessor;
    }
}