package NeuralNetwork;

import org.deeplearning4j.base.MnistFetcher;
import org.deeplearning4j.util.MathUtils;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.dataset.DataSet;
import org.nd4j.linalg.factory.Nd4j;

import java.io.IOException;
import java.util.Arrays;
import java.util.Random;

/**
 * Created by Maaarty on 4.3.2017.
 */
public class GomokuDataFetcherSmall extends BaseDataFetcher {

   /*  public static final int NUM_EXAMPLES = 10000;//9984;
    public static final int NUM_EXAMPLES_TEST = 5000;//4864;
 /* */

   public static final int NUM_EXAMPLES = 5;
     public static final int NUM_EXAMPLES_TEST = 5;
   // */
    // protected static final String TEMP_ROOT = System.getProperty("user.home");
     //protected static final String MNIST_ROOT = TEMP_ROOT + File.separator + "Mnist" + File.separator;

     protected transient GomokuManagerSmall man;
     protected boolean binarize = true;
     protected boolean train;
     protected int[] order;
     protected Random rng;
     protected boolean shuffle;


     /**
      * Constructor telling whether to binarize the dataset or not
      *
      * @param binarize whether to binarize the dataset or not
      * @throws IOException
      */
    public GomokuDataFetcherSmall(boolean binarize) throws IOException {
        this(binarize, true, true, System.currentTimeMillis(), 0, 5);
    }

    public GomokuDataFetcherSmall(boolean binarize, boolean train, boolean shuffle, long rngSeed, int k, int t) throws IOException {
        if (!mnistExists()) {
            new MnistFetcher().downloadAndUntar();
        }
        String images;
        String labels;
        if (k == -2) {
            labels = "TestingFileLabelOfNeuralNetwork.txt";
            images = "TestingFileOfNeuralNetwork.txt";
            totalExamples = NUM_EXAMPLES_TEST;

        } else {
            if (train) {
                labels = "dataforneuralnetwork" + t + "/trainingDataGomokuNextExpectingMove" + k + ".txt";
                images = "dataforneuralnetwork" + t + "/trainingDataGomokuPartialGames" + k + ".txt";
                totalExamples = NUM_EXAMPLES;
            } else {
                labels = "testingFileLabels.txt";
                images = "testingFileGames.txt";
                totalExamples = NUM_EXAMPLES_TEST;
            }
        }

        try {
            man = new GomokuManagerSmall(images, labels, train);
        } catch (Exception e) {
            //FileUtils.deleteDirectory(new File(MNIST_ROOT));
           // new MnistFetcher().downloadAndUntar();
            //man = new MnistManager(images, labels, train);
            System.out.println(e.getMessage());
        }

        numOutcomes = 225;
        this.binarize = binarize;
        cursor = 0;
        inputColumns = man.getImages().getEntryLength();
        this.train = train;
        this.shuffle = shuffle;

        if (train) {
            order = new int[NUM_EXAMPLES];
        } else {
            order = new int[NUM_EXAMPLES_TEST];
        }
        for (int i = 0; i < order.length; i++) order[i] = i;
        rng = new Random(rngSeed);
        reset();    //Shuffle order
    }

    private boolean mnistExists() {
        //Check 4 files:
       /* File f = new File(MNIST_ROOT, MnistFetcher.trainingFilesFilename_unzipped);
        if (!f.exists()) return false;
        f = new File(MNIST_ROOT, MnistFetcher.trainingFileLabelsFilename_unzipped);
        if (!f.exists()) return false;
        f = new File(MNIST_ROOT, MnistFetcher.testFilesFilename_unzipped);
        if (!f.exists()) return false;
        f = new File(MNIST_ROOT, MnistFetcher.testFileLabelsFilename_unzipped);
        if (!f.exists()) return false;*/
        return true;
    }

    public GomokuDataFetcherSmall() throws IOException {
        this(true);
    }

    @Override
    public void fetch(int numExamples) {
        if (!hasMore()) {
            throw new IllegalStateException("Unable to getFromOrigin more; there are no more images");
        }

        // zmenil jsem float na int
        float[][] featureData = new float[numExamples][0];
        float[][] labelData = new float[numExamples][0];

        int actualExamples = 0;
        for (int i = 0; i < numExamples; i++, cursor++) {
            if (!hasMore()) break;

            float[] img = man.readImageUnsafe(order[cursor]);
            float[] label = man.readLabel(order[cursor]);

            featureData[actualExamples] = img;
            labelData[actualExamples] = label;



            actualExamples++;
        }

        if (actualExamples < numExamples) {
            featureData = Arrays.copyOfRange(featureData, 0, actualExamples);
            labelData = Arrays.copyOfRange(labelData, 0, actualExamples);
        }

        INDArray features = Nd4j.create(featureData);
        INDArray labels = Nd4j.create(labelData);
        curr = new DataSet(features, labels);
    }

    @Override
    public void reset() {
        cursor = 0;
        curr = null;
        if (shuffle) MathUtils.shuffleArray(order, rng);
    }

    public GomokuManagerSmall returnMen()
    {
        return man;
    }

    @Override
    public DataSet next() {
        DataSet next = super.next();
        return next;
    }
}
