package NeuralNetwork;

import java.io.*;
import java.util.ArrayList;

/**
 * Created by Maaarty on 7.3.2017.
 */
public class GomokuImageFileSmall extends GomokuDbFile {
    private int rows;
    private int cols;

    /**
     * Creates new MNIST database image file ready for reading.
     *
     * @param name
     *            the system-dependent filename
     * @param mode
     *            the access mode
     * @throws IOException
     * @throws FileNotFoundException
     */
    public GomokuImageFileSmall(String name, String mode) throws  IOException {
        super(name, mode);

        // read header information
        rows = 15;
        cols = 15;
    }

    /**
     * Reads the image at the current position.
     *
     * @return matrix representing the image
     * @throws IOException
     */
    public int[][] readImage() throws IOException {
        int[][] dat = new int[getRows()][getCols()];
        for (int i = 0; i < getCols(); i++) {
            for (int j = 0; j < getRows(); j++) {
                dat[i][j] = readUnsignedByte();
            }
        }
        return dat;
    }

    private int giveMeSmallSquare(float[] label)
    {
        float max = 0;
        int newPosition = 0;
        for (int i = 0; i < label.length; i++) {
            if (label[i] > max) {
                max = label[i];
                newPosition = i;
            }
        }

        int retValue;
        // It is for 7x7 Neural Network
        int posOfCol = (newPosition % 15);
        if (newPosition < 75) {
            if (posOfCol < 5) {
                retValue = 0;
            } else {
                if (posOfCol > 9) {
                    retValue = 8;
                } else {
                    retValue = 4;
                }
            }
        } else {
            if (newPosition > 149) {
                if (posOfCol < 5) {
                    retValue = 120;
                } else {
                    if (posOfCol > 9) {
                        retValue = 128;
                    } else {
                        retValue = 124;
                    }
                }
            } else {
                if (posOfCol < 5) {
                    retValue = 60;
                } else {
                    if (posOfCol > 9) {
                        retValue = 68;
                    } else {
                        retValue = 64;
                    }
                }
            }
        }

        // it is for 5x5 Neural Network
        //int posOfCol = (newPosition % 15) / 5;
        //int posOfRow = (newPosition / 15) / 5;
        //int retValue = posOfRow * 15 * 5 + posOfCol * 5;
        return retValue;


    }


    private float[] giveMeSmallArray2(float[] bigArray, int newPoss)
    {
        float[] smallArray = new float[49];
        int shifter = newPoss;
        for (int i = 0; i < 7; i++) {
            for (int j = 0; j < 7; j++) {
                smallArray[i * 7 + j] = bigArray[shifter + i * 15 + j];
            }
        }
        return smallArray;
    }
    private int[] giveMeSmallArray2(int[] bigArray, int newPoss)
    {
        int[] smallArray = new int[49];
        int shifter = newPoss;
        for (int i = 0; i < 7; i++) {
            for (int j = 0; j < 7; j++) {
                smallArray[i * 7 + j] = bigArray[shifter + i * 15 + j];
            }
        }
        return smallArray;
    }


    private float[] giveMeSmallArray(float[] bigArray, int newPoss)
    {
        float[] smallArray = new float[25];
        int shifter = newPoss;
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                smallArray[i * 5 + j] = bigArray[shifter + i * 15 + j];
            }
        }
        return smallArray;
    }
    private int[] giveMeSmallArray(int[] bigArray, int newPoss)
    {
        int[] smallArray = new int[25];
        int shifter = newPoss;
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                smallArray[i * 5 + j] = bigArray[shifter + i * 15 + j];
            }
        }
        return smallArray;
    }

    /** Read the specified number of images from the current position, to a byte[nImages][rows*cols]
     * Note that MNIST data set is stored as unsigned bytes; this method returns signed bytes without conversion
     * (i.e., same bits, but requires conversion before use)
     * @param nImages Number of images
     */
    public ArrayList<int[]> readImagesUnsafe(int nImages, ArrayList<float[]> labelsArr) throws IOException{
        ArrayList<int[]> out = new ArrayList<int[]>(nImages);
        for( int i=0; i<nImages; i++){
            float[] fullLabel = labelsArr.get(i);
            int newPos = giveMeSmallSquare(fullLabel);
            float[] smallLabel = giveMeSmallArray2(fullLabel, newPos);
            labelsArr.set(i, smallLabel);
           /* if ((i % 10000) == 0 && i != 0) {
                System.out.println("We have loaded " + i + " examples!");
            }*/
            int[] line = new int[rows*cols];
            String[] newLine = readLine().split(" ");
            for (int j = 0; j < newLine.length; j += 2) {
                line[(Integer.valueOf(newLine[j]))] = 2;
            }
            for (int j = 1; j < newLine.length; j += 2) {
                line[(Integer.valueOf(newLine[j]))] = 1;
            }
            line = giveMeSmallArray2(line, newPos);
            out.add(line);
        }
        super.close();
        return out;
    }

    /**
     * Move the cursor to the next image.
     *
     * @throws IOException
     */
    public void nextImage() throws IOException {
        super.next();
    }

    private void readingfromfile(String nameoffile) {

        BufferedReader fr = null;
        try {
            File otevreniSouboru = new File(nameoffile);
            fr = new BufferedReader(new FileReader(otevreniSouboru));
            StringBuilder builder = new StringBuilder();
            String radka;
            String newdata;
            boolean ukonceni = false;

            while(!ukonceni) {
                // newdata = "";
                radka = fr.readLine();
                while("".equals(radka)) {
                    radka = fr.readLine();
                }
                if( radka == null){
                    ukonceni = true;
                    continue;
                }
                if("".equals(radka)){
                    continue;
                }

                while(radka != null) {
                    if("".equals(radka)){
                        break;
                    }
                    radka = fr.readLine();
                }
                while("".equals(radka)) {
                    radka = fr.readLine();
                }
                while(radka != null ) {
                    if("".equals(radka)){
                        break;
                    }
                    builder.append(" ").append(radka);
                    //newdata += " " + radka;
                    radka = fr.readLine();
                }
                newdata = builder.toString();
                //makedataforhashmapfromgame(newdata.split(" "));
                builder = new StringBuilder();
                //makedataforhashmapfromgame(newdata.split(" "));
                //  System.out.println(newdata);

            }

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
            System.out.println("Je to tu 96");

        }
    }



    /**
     * Move the cursor to the previous image.
     *
     * @throws IOException
     */
    public void prevImage() throws IOException {
        super.prev();
    }

    @Override
    protected int getMagicNumber() {
        return 2051;
    }

    /**
     * Number of rows per image.
     *
     * @return int
     */
    public int getRows() {
        return rows;
    }

    /**
     * Number of columns per image.
     *
     * @return int
     */
    public int getCols() {
        return cols;
    }

    @Override
    public int getEntryLength() {
        return cols * rows;
    }

    @Override
    public int getHeaderSize() {
        return super.getHeaderSize() + 8; // to more integers - rows and columns
    }
}
