package NeuralNetwork;

/**
 * Created by Maaarty on 16.3.2017.
 */

import org.deeplearning4j.nn.api.Model;
import org.deeplearning4j.nn.multilayer.MultiLayerNetwork;
import org.deeplearning4j.optimize.api.IterationListener;
import org.deeplearning4j.util.ModelSerializer;
import org.nd4j.linalg.dataset.api.iterator.DataSetIterator;

import java.io.File;

/**
 * Score iteration listener
 *
 * @author Adam Gibson
 */
public class ScoreIterationListenerSmall implements IterationListener {
    private int printIterations = 10;
    //private static final Logger log = LoggerFactory.getLogger(ScoreIterationListener.class);
    private boolean invoked = false;
    private long iterCount = 0;
    private double theBestScore;
    NeuralNetworkSmall neuralNetwork;
    int localNumberOfIterations;
    int nFilters;
    double learningRate;
    double biasLearningRate;
    double lrPolicyDecayRate;
    Evaluation eval;
    DataSetIterator gomokuTest;
    /**
     * @param printIterations    frequency with which to print scores (i.e., every printIterations parameter updates)
     */
    public ScoreIterationListenerSmall(int printIterations, NeuralNetworkSmall net, int nFilters, double learningRate, double biasLearningRate, double lrPolicyDecayRate,
                                       Evaluation eval, DataSetIterator gomokuTest) {
        this.printIterations = printIterations;
        theBestScore = Integer.MAX_VALUE;
        neuralNetwork = net;
        localNumberOfIterations = 10000 / neuralNetwork.sizeOfBatch;
        localNumberOfIterations += 1;
        this.nFilters = nFilters;
        this.learningRate = learningRate;
        this.biasLearningRate = biasLearningRate;
        this.lrPolicyDecayRate = lrPolicyDecayRate;
        this.eval = eval;
        this.gomokuTest = gomokuTest;
    }

    /** Default constructor printing every 10 iterations */
    public ScoreIterationListenerSmall() {
    }

    @Override
    public boolean invoked(){ return invoked; }

    @Override
    public void invoke() { this.invoked = true; }

    double localScore = 0;
    int pointer = 0;

    @Override
    public void iterationDone(Model model, int iteration) {
        if(printIterations <= 0)
            printIterations = 1;
        if(iterCount % printIterations == 0) {
            invoke();
            double result = model.score();
            if (theBestScore >= result) {
                theBestScore = result;

                saveTheModel((MultiLayerNetwork) model);
            }
            if (result < 200 && result >= 0) {
                localScore += result;
            } else {
                localScore += 200;
            }
            pointer += 1;
            if ( localNumberOfIterations == pointer) {
                neuralNetwork.localValue = localScore / localNumberOfIterations;
                localScore = 0;
                pointer = 0;
            }
            //log.info("Score at iteration " + iterCount + " is " + result);
            System.out.println("Score at iteration " + iterCount + " is " + result);
         /*   eval = new Evaluation(225);
            while (gomokuTest.hasNext()) {
                DataSet ds = gomokuTest.next();
                INDArray output = neuralNetwork.model.output(ds.getFeatureMatrix(), false);
                eval.eval(ds.getLabels(), output);
            }
            System.out.println( eval.stats2(true));
            gomokuTest.reset();*/

        }
        iterCount++;
    }

    private boolean saveTheModel(MultiLayerNetwork net)
    {
        try {
            //Save the model
            File locationToSave = new File("MyMultiLayerNetworkTopInSeries" + learningRate + biasLearningRate + lrPolicyDecayRate + nFilters + ".zip");      //Where to save the network. Note: the file is in .zip format - can be opened externally
            boolean saveUpdater = true;                                             //Updater: i.e., the state for Momentum, RMSProp, Adagrad etc. Save this if you want to train your network more in the future
            ModelSerializer.writeModel(net, locationToSave, saveUpdater);
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
            System.out.println("It have failed when saving the model.");
            System.out.println("It is here 96");

        }
        return true;
    }


}